require 'logger'
require 'bcrypt'
require 'celluloid/autostart'
require 'action_mailer'
require 'premailer'
require 'mongoid'
require 'active_support/core_ext/object/blank'
require 'active_support/core_ext/string/strip'
require 'active_support/core_ext/hash/slice'
require 'active_support/core_ext/hash/except'
require 'active_support/core_ext/date/calculations'
require 'active_support/core_ext/time/calculations'
require 'active_support/core_ext/numeric/time'
require 'active_support/core_ext/enumerable'

require "snowman-io/version"
require "snowman-io/utils"
require "snowman-io/web"
require "snowman-io/api"
require "snowman-io/options"
require "snowman-io/launcher"
require "snowman-io/cli"
require "snowman-io/web_server"
require "snowman-io/aggregate"
require "snowman-io/loop/main"
require "snowman-io/loop/checks"
require "snowman-io/loop/spiders"
require "snowman-io/snow_mailer"
require "snowman-io/migration"
require "snowman-io/spider/internal_mongo"
require "snowman-io/spider/mps"
require "snowman-io/models/concerns/tokenable"
require "snowman-io/models/user"
require "snowman-io/models/app"
require "snowman-io/models/metric"
require "snowman-io/models/check"
require "snowman-io/models/data_point"
require "snowman-io/models/aggregation"
require "snowman-io/models/setting"
require "snowman-io/models/deleted"
require "snowman-io/models/following"

ActionMailer::Base.raise_delivery_errors = true
ActionMailer::Base.view_paths = File.dirname(__FILE__) + "/snowman-io/views"

if ENV["TEST_MODE"].to_i == 1
  ActionMailer::Base.delivery_method = :test
elsif ENV["DEV_MODE"].to_i == 1
  Object.send(:remove_const, :Rails)
  require "letter_opener"
  ActionMailer::Base.add_delivery_method :letter_opener,
    LetterOpener::DeliveryMethod,
    :location => File.expand_path('../../tmp/letter_opener', __FILE__)
  ActionMailer::Base.delivery_method = :letter_opener
else
  ActionMailer::Base.delivery_method = :smtp
  ActionMailer::Base.smtp_settings = {
    :address   => ENV["MAILGUN_SMTP_SERVER"],
    :port      => ENV["MAILGUN_SMTP_PORT"],
    :authentication => :plain,
    :user_name      => ENV["MAILGUN_SMTP_LOGIN"],
    :password       => ENV["MAILGUN_SMTP_PASSWORD"],
    :enable_starttls_auto => true
  }
end

module SnowmanIO
  BASE_URL_KEY = "base_url"
  FORCE_SSL_KEY = "force_ssl"

  def self.logger
    @logger ||= Logger.new(STDERR)
  end

  def self.unpacked?
    User.count > 0
  end
end

Mongoid.load!(File.expand_path("../config/mongoid.yml", __FILE__), :production)
