module SnowmanIO
  module API
    class Invites < Grape::API
      namespace :users do
        params do
          requires :email, type: String
        end
        post "invite" do
          authenticate!
          if User.where(email: permitted_params[:email]).empty?
            user = User.create!(
              email: permitted_params[:email],
              password: SecureRandom.hex,
              status: "wait_invite"
            )
            user.invite!(current_user)
            { user: user }
          else
            status 400
            { message: "wrong invite request" }
          end
        end

        post ":id/resend_invite" do
          authenticate!
          user = User.where(status: "wait_invite", id: params[:id]).first

          if user && user.invite_token.present?
            user.invite!(current_user)
            { user: user }
          else
            status 400
            { message: "wrong invite resend request" }
          end
        end

        post ":id/cancel_invite" do
          authenticate!
          user = User.where(status: "wait_invite", id: params[:id]).first

          if user && user.invite_token.present?
            user.destroy
            { status: "ok" }
          else
            status 400
            { message: "wrong invite cancel request" }
          end
        end

        params do
          requires :token, type: String
        end
        post "check_invite" do
          {
            correct: !!(
              permitted_params[:token].present? &&
              User.where(status: "wait_invite", invite_token: permitted_params[:token]).first
            )
          }
        end

        params do
          requires :token, type: String
          requires :name, type: String
          requires :password, type: String
        end
        post "accept_invite" do
          token = permitted_params[:token]
          if token.present? && (user = User.where(status: "wait_invite", invite_token: token).first)
            user.update_attributes!(
              name: permitted_params[:name],
              password: permitted_params[:password],
              status: "active",
              invite_token: ""
            )
            { user: user }
          else
            status 400
            { message: "SnowmanIO bad accept_invite request" }
          end
        end
      end
    end
  end
end
