module SnowmanIO
  module API
    class Users < Grape::API
      namespace :users do
        desc "User signup"
        params do
          requires :user, type: Hash do
            requires :name, type: String
            requires :password, type: String
            requires :email, type: String
          end
        end
        post do
          unless SnowmanIO.unpacked?
            user = User.create!(permitted_params[:user].to_h)
            { user: user }
          else
            status 400
            { message: "SnowmanIO is unpacked already" }
          end
        end

        desc "User Signin"
        params do
          requires :user, type: Hash do
            requires :email, type: String
            optional :password, type: String
          end
        end
        post "login" do
          email = permitted_params[:user][:email]
          password = permitted_params[:user][:password]
          if (user = User.where(status: "active", email: email).first) && password.present? && user.authenticate(password)
            { token: user.authentication_token, email: user.email, user_id: user.id.to_s }
          else
            status 400
            { message: "Wrong email or password" }
          end
        end

        params do
          requires :email, type: String
        end
        post "restore_password" do
          if user = User.where(status: "active", email: permitted_params[:email]).first
            user.restore_password!
            { status: "ok" }
          else
            status 400
            { message: "Unknown email" }
          end
        end

        params do
          requires :token, type: String
        end
        post "check_pass_token" do
          {
            correct: !!(
              permitted_params[:token].present? &&
              User.where(status: "active", restore_pass_token: permitted_params[:token]).first
            )
          }
        end

        params do
          requires :token, type: String
          requires :password, type: String
        end
        post "reset_password" do
          token = permitted_params[:token]
          if token.present? && (user = User.where(status: "active", restore_pass_token: token).first)
            user.update_attributes!(
              password: permitted_params[:password],
              restore_pass_token: ""
            )
            { user: user }
          else
            status 400
            { message: "SnowmanIO bad reset_password request" }
          end
        end

        post ":id/destroy" do
          authenticate!
          if user = User.where(id: params[:id]).first
            Extra::Meteor.model_destroy(user)
            { status: "ok"}
          else
            status 400
            { status: "failed", message: "Wrong id" }
          end
        end
      end
    end
  end
end
