require 'premailer'

module SnowmanIO
  class SnowMailer < ActionMailer::Base
    default(
      template_path: "snow_mailer",
      from: "no-reply@example.com"
    )

    def restore_password(user, base_url)
      @url = base_url + "/restore_password/" + user.restore_pass_token
      @user = user
      mail(
        to: user.email,
        subject: "SnowmanIO: password restore"
      ) do |format|
        format.html { 
          Premailer.new(render(:"snow_mailer/restore_password", layout: "main"), {
            css: [
              File.expand_path('../views/layouts/styles.css', __FILE__),
              File.expand_path('../views/layouts/custom.css', __FILE__)
            ],
            with_html_string: true
          }).to_inline_css
        }
      end
    end

    def send_invite(user, base_url, by)
      @url = base_url + "/invite/" + user.invite_token
      @by = by
      mail(
        to: user.email,
        subject: "SnowmanIO: invite"
      ) do |format|
        format.html { 
          Premailer.new(render(:"snow_mailer/send_invite", layout: "main"), {
            css: [
              File.expand_path('../views/layouts/styles.css', __FILE__),
              File.expand_path('../views/layouts/custom.css', __FILE__)
            ],
            with_html_string: true
          }).to_inline_css
        }
      end
    end
    
    def check_triggered(check, at, base_url, to, danger)
      @check = check
      @at = at
      @base_url = base_url
      @danger = danger
      mail(
        to: to,
        subject: "SnowmanIO: failed check at #{at.strftime("%Y-%m-%d %H:%M:%S")}"
      ) do |format|
        format.html { 
          Premailer.new(render(:"snow_mailer/check_triggered", layout: "main"), {
            css: [
              File.expand_path('../views/layouts/styles.css', __FILE__),
              File.expand_path('../views/layouts/custom.css', __FILE__)
            ],
            with_html_string: true
          }).to_inline_css
        }
      end
    end
  end
end
