require 'sinatra'
require 'rack/ssl'

module SnowmanIO
  class Web < Sinatra::Base
    set :public_folder, File.dirname(__FILE__) + "/ui"
    set :views, File.dirname(__FILE__) + "/ui"

    use Rack::SSL, exclude: Proc.new { |env|
      ENV['DEV_MODE'].to_i == 1 || !Setting.force_ssl?
    }

    before do
      unless Setting.get(SnowmanIO::BASE_URL_KEY).present?
        Setting.set(SnowmanIO::BASE_URL_KEY, request.base_url)
      end
    end

    get "/ping" do
      "PONG"
    end

    # Ember application
    get '/*' do
      # Dont cache index page
      # http://stackoverflow.com/questions/49547/making-sure-a-web-page-is-not-cached-across-all-browsers
      headers["Cache-Control"] = "no-cache, no-store, must-revalidate" # HTTP 1.1.
      headers["Pragma"] = "no-cache" # HTTP 1.0.
      headers["Expires"] = "0" # Proxies.

      # replace version only on production.
      if ENV["DEV_MODE"].to_i == 1
        File.read(File.expand_path("../ui/index.html", __FILE__))
      else
        File.read(File.expand_path("../ui/index.html", __FILE__)).sub("__VERSION__", SnowmanIO::VERSION)
      end
    end
  end
end
